pro shapelets_deconvolve, structure_a, structure_b, $
                          SILENT=silent,            $
                          NOHISTORY=nohistory,      $
                          EXTEND=extend,            $
                          MAINTAIN=maintain

;$Id: shapelets_deconvolve.pro, v2$
;
; Copyright  2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;      SHAPELETS_DECONVOLVE
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Deconvolves one image from another (e.g. a PSF).
;      This is a simple wrapper for one option within shapelets_convolve.pro,
;      where all the work is really done.
;
; INPUTS:
;      STRUCTURE_A - A shapelet decomp or shapecat structure.
;      STRUCTURE_B - A shapelet decomp structure representing the second image,
;                 or the PSF model (the PSF can be either structure since the
;                 operation is symmetric).
;
; OPTIONAL INPUTS:
;      None.
;
; KEYWORD PARAMETERS:
;      None.
;
; EXAMPLE:
;      shapelets_deconvolve,decomp,psf
;
; OUTPUTS:
;      Replaces first input with a deconvolved version. Second input is unchanged.
;
; MODIFICATION HISTORY:
;      Apr 07 - Written by Richard Massey.
;-

COMPILE_OPT idl2

shapelets_convolve, structure_a, structure_b, /DECONVOLVE,         $
                                              SILENT=silent,       $
                                              NOHISTORY=nohistory, $
                                              EXTEND=extend,       $
                                              MAINTAIN=maintain
                    
end
